<?php

namespace Drupal\widget_provider_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\widget_provider_api\WidgetProviderBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\widget_provider_api\WidgetProviderPluginManager;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for WidgetProviderAPI routes.
 */
class WidgetProviderApiController extends ControllerBase {

  /**
   * The widget plugin manager.
   *
   * @var \Drupal\widget_provider_api\WidgetProviderPluginManager
   */
  protected $widgetProviderPluginManager;

  /**
   * Constructs the WidgetApiController object.
   *
   * @param \Drupal\widget_provider_api\WidgetProviderPluginManager $widgetProviderPluginManager
   *   The module handler service.
   */
  public function __construct(WidgetProviderPluginManager $widgetProviderPluginManager) {
    $this->widgetProviderPluginManager = $widgetProviderPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): object {
    return new static(
      $container->get('plugin.manager.widget_provider')
    );
  }

  /**
   * Build the response.
   *
   * @param string $group
   *   The widget group.
   * @param string $widgetProvider
   *   The widget provider.
   *
   * @return array|\Symfony\Component\HttpFoundation\Response
   *   The response array or Response object.
   */
  public function build($group, $widgetProvider) {
    try {
      if (preg_match('/[^a-z\-0-9]/i', $group)) {
        throw new \Exception('Invalid group name given. Only a-z, 0-9, and - are allowed.');
      }
      if (preg_match('/[^a-z\-0-9]/i', $widgetProvider)) {
        throw new \Exception('Invalid widget provider name given. Only a-z, 0-9, and - are allowed.');
      }
      $request = \Drupal::request();
      // Catch all exceptions and return a 404 instead:
      /** @var \Drupal\widget_provider_api\WidgetProviderInterface $widgetProviderInstance*/
      $widgetProviderInstance = $this->widgetProviderPluginManager->createInstance("$group:$widgetProvider", [
        'queryParameters' => $request->query->all(),
        'referer' => $request->headers->get('referer'),
      ]);
      return $widgetProviderInstance->build();
    }
    // Show a not found exception, if any error occurs:
    catch (\Exception $e) {
      throw new NotFoundHttpException();
    }
  }

  /**
   * Build the widget provider list.
   *
   * @return array
   *   Render array for the widget provider list.
   */
  public function buildList() {
    $plugin_definitions = $this->widgetProviderPluginManager->getDefinitions();
    $rows = [];

    foreach ($plugin_definitions as $plugin_id => $definition) {
      $explodedPluginId = WidgetProviderBase::explodePluginId($plugin_id);
      $rows[] = [
        'label' => $definition['label'],
        'group' => $explodedPluginId[0],
        'id' => $explodedPluginId[1],
        'requiredParameters' => !empty($definition['requiredParameters']) ? implode(', ', $definition['requiredParameters']) : '-',
        'optionalParameters' => !empty($definition['optionalParameters']) ? implode(', ', $definition['optionalParameters']) : '-',
        'allowedReferrers' => !empty($definition['allowedReferrers']) ? implode(', ', $definition['allowedReferrers']) : '-',
        'secret' => $definition['secret'] ?? '-',
        'generatedUrl' => $definition['class']::generateUrl($definition['secret'], $definition['id'], $definition['requiredParameters'] ?? [], $definition['optionalParameters'] ?? []),
      ];
    }

    return [
      '#theme' => 'widget_provider_api_widget_provider_list',
      '#providers' => $rows,
    ];
  }

}
