<?php

declare(strict_types=1);

namespace Drupal\widget_provider_api;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\BareHtmlPageRenderer;
use Drupal\Core\Url;
use Drupal\widget_provider_api\Exceptions\NoRegisteredWidgetParameterException;
use Drupal\widget_provider_api\Exceptions\WidgetValidationUnallowedReferrerException;
use Drupal\widget_provider_api\Exceptions\WidgetValidationWrongChecksumException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for widget plugins.
 */
abstract class WidgetProviderBase extends PluginBase implements WidgetProviderInterface, ContainerFactoryPluginInterface {

  /**
   * The query parameters.
   *
   * @var array
   */
  protected array $queryParameters = [];

  /**
   * Constructor.
   *
   * @param string $pluginId
   *   The plugin ID for the widget.
   * @param mixed $pluginDefinition
   *   The plugin implementation definition.
   * @param array $queryParameters
   *   The query parameters.
   * @param string|null $referer
   *   The referer.
   */
  public function __construct($pluginId, $pluginDefinition, array $queryParameters = [], ?string $referer = NULL) {
    // First off, validate all parameters:
    $this->validateAll($referer, $queryParameters, $pluginDefinition);
    // All valid. Construct the widget provider:
    parent::__construct([], $pluginId, $pluginDefinition);
    // We don't want to have the secret in our query parameters:
    unset($queryParameters['h']);
    $this->queryParameters = $queryParameters;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): WidgetProviderInterface {
    return new static($plugin_id, $plugin_definition, $configuration['queryParameters'], $configuration['referer']);
  }

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  abstract protected function buildContent();

  /**
   * {@inheritdoc}
   */
  public function build() {
    $buildContent = $this->buildContent();
    if (is_array($buildContent)) {
      // The result was a Drupal render array, we have to render it, otherwise
      // it would be output in site context (template):
      // @todo LATER think about how we could inject the response here and how
      // to configure the expected response type (with or without theme...)
      // @see https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Render%21HtmlResponse.php/class/HtmlResponse/8.2.x
      // Render in bare page without template around:
      $attachments = \Drupal::service('html_response.attachments_processor');
      $renderer = \Drupal::service('renderer');
      $bareHtmlPageRenderer = new BareHtmlPageRenderer($renderer, $attachments);
      // @see https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Render%21BareHtmlPageRendererInterface.php/interface/BareHtmlPageRendererInterface/8.2.x
      $response = $bareHtmlPageRenderer->renderBarePage($buildContent, $this->label(), 'markup');
      // @todo This still contains CSS from theme. We have to decide what we really want here and how to control:
      // ALTERNATIVE WITHOUT ANY WRAPPING HTML:
      // $response = new HtmlResponse();
      // // Render RenderArray as response:
      // $response->setContent($buildContent);
      return $response;
    }
    else {
      return $buildContent;
    }
  }

  /**
   * Validates all important Parameters.
   *
   * @return bool
   *   TRUE if all parameters are valid, FALSE otherwise.
   */
  protected function validateAll(?string $referer, array $queryParameters, array $pluginDefinition): bool {
    if (!$this->validateRequiredParameters($queryParameters, $pluginDefinition)) {
      throw new \Exception('Missing required parameters!');
    }
    if (!$this->validateAllowedReferer($referer, $pluginDefinition)) {
      throw new WidgetValidationUnallowedReferrerException();
    }
    if (!$this->validateChecksum($queryParameters, $pluginDefinition)) {
      throw new WidgetValidationWrongChecksumException();
    }
    return TRUE;
  }

  /**
   * Validates the required parameters.
   */
  private function validateRequiredParameters(array $queryParameters, array $pluginDefinition): bool {
    $requiredParameters = array_intersect_key($queryParameters, array_flip($pluginDefinition['requiredParameters'] ?? []));
    if (!empty($pluginDefinition['requiredParameters']) && empty($requiredParameters)) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Validates the allowed referrers of the widget.
   *
   * @return bool
   *   TRUE if the referrer is allowed, FALSE otherwise.
   */
  private function validateAllowedReferer(?string $referer, array $pluginDefinition): bool {
    if (empty($pluginDefinition['allowedReferrers'])) {
      return TRUE;
    }
    return in_array($referer, $pluginDefinition['allowedReferrers']);
  }

  /**
   * Validates the Widget Checksum.
   *
   * @return bool
   *   TRUE if the checksum is valid, FALSE otherwise.
   */
  private function validateChecksum($queryParameters, $pluginDefinition): bool {
    // No secret was set, this is a public widget:
    if (empty($pluginDefinition['secret'])) {
      return TRUE;
    }
    // There was not secret given, fail early:
    if (empty($queryParameters['h'])) {
      return FALSE;
    }
    $incomingChecksum = $queryParameters['h'];
    // Unset the secret, so we don't accidentally hash it:
    unset($queryParameters['h']);
    $builtChecksum = self::generateChecksum($pluginDefinition['secret'], $pluginDefinition['id']);

    return $incomingChecksum === $builtChecksum;
  }

  /**
   * Generates a checksum based on the provided query parameters and secret.
   *
   * @param string $secret
   *   The secret key used to generate the checksum.
   * @param string $pluginId
   *   The plugin ID for which the checksum is generated.
   *
   * @return string
   *   The generated checksum as a string.
   */
  protected static function generateChecksum($secret, $pluginId): string {
    $explodedPluginId = self::explodePluginId($pluginId);
    /** @var \Drupal\Core\Routing\RouteProvider $routeProvider */
    // @todo Find a better way to get the route path:
    $routePath = '/widget-provider/' . $explodedPluginId[0] . '/' . $explodedPluginId[1];
    $password = $secret . $routePath;
    return hash('sha256', $password);
  }

  /**
   * {@inheritdoc}
   */
  public static function generateUrl($secret, $pluginId, $requiredParameters = [], $optionalParameters = []): string {
    $processedRequiredParams = array_map(function () {
      return 'SET_HERE';
    }, array_flip($requiredParameters));
    $processedOptionalParams = array_map(function () {
      return 'OPTIONALLY_SET_HERE';
    }, array_flip($optionalParameters));
    $checksum = self::generateChecksum($secret, $pluginId);
    $explodedPluginId = self::explodePluginId($pluginId);
    return Url::fromRoute('widget_provider_api.widget_provider_route', [
      'group' => $explodedPluginId[0],
      'widgetProvider' => $explodedPluginId[1],
    ], [
      'query' => array_merge(['h' => $checksum], $processedRequiredParams, $processedOptionalParams),
    ])->setAbsolute()->toString();
  }

  /**
   * {@inheritdoc}
   */
  public static function explodePluginId(string $pluginId): array {
    $parts = explode(':', $pluginId);
    if (count($parts) !== 2) {
      throw new \InvalidArgumentException('Invalid plugin ID format. It should be in the format "group:widget_provider".');
    }
    return $parts;
  }

  /**
   * {@inheritdoc}
   */
  public function getAllowedReferrers(): array {
    return $this->pluginDefinition['allowedReferrers'];
  }

  /**
   * {@inheritdoc}
   */
  public function getRequiredParameters(): array {
    return array_intersect_key($this->queryParameters, array_flip($this->pluginDefinition['requiredParameters']));
  }

  /**
   * {@inheritdoc}
   */
  public function getOptionalParameters(): array {
    return array_intersect_key($this->queryParameters, array_flip($this->pluginDefinition['optionalParameters']));
  }

  /**
   * {@inheritdoc}
   */
  public function getAllParameters(): array {
    $requiredParas = $this->getRequiredParameters();
    $optionalParas = $this->getOptionalParameters();
    return array_merge($requiredParas, $optionalParas);
  }

  /**
   * {@inheritdoc}
   */
  public function getParameterValue($parameterKey) {
    if (!array_key_exists($parameterKey, $this->getAllParameters())) {
      throw new NoRegisteredWidgetParameterException($parameterKey . ' is not a registered widget parameter. You can only retrieve registered required or optional parameters. Did you forget to list them in this widget?');
    }
    return $this->getAllParameters()[$parameterKey];
  }

}
