<?php

declare(strict_types=1);

namespace Drupal\widget_provider_api;

/**
 * Interface for widget plugins.
 */
interface WidgetProviderInterface {

  /**
   * Returns the translated plugin label.
   */
  public function label(): string;

  /**
   * Explodes the plugin ID.
   *
   * Explodes the plugin ID (group:widget_provider) into group (baseId) and
   * widget_provider (derivate id).
   */
  public static function explodePluginId(string $plugin_id): array;

  /**
   * Generates a URL leading to the widget.
   *
   * @param string $secret
   *   The secret used to authenticate the widget provider.
   * @param string $pluginId
   *   The plugin ID of the widget provider.
   * @param array $requiredParameters
   *   An array of required parameters.
   * @param array $optionalParameters
   *   An array of optional parameters.
   *
   * @return string
   *   The generated URL.
   */
  public static function generateUrl($secret, $pluginId, $requiredParameters = [], $optionalParameters = []): string;

  /**
   * Retrieve required Parameters.
   *
   * @return array
   *   An array of required parameters.
   */
  public function getRequiredParameters(): array;

  /**
   * Retrieve optional Parameters.
   *
   * @return array
   *   An array of optional parameters.
   */
  public function getOptionalParameters(): array;

  /**
   * Retrieve all Parameters.
   *
   * @return array
   *   An array of all parameters.
   */
  public function getAllParameters(): array;

  /**
   * Retrieve all Allowed Widget Referrers.
   *
   * @return array
   *   An array of allowed referrers.
   */
  public function getAllowedReferrers(): array;

  /**
   * Build the Response.
   *
   * @return array|\Symfony\Component\HttpFoundation\Response
   *   The response array or Response object.
   */
  public function build();

  /**
   * Returns the value of the given registered parameter key.
   *
   * @param string $parameterKey
   *   The parameter key.
   *
   * @return mixed
   *   The parameter value.
   *
   * @throws \Drupal\widget_provider_api\Exception\NoRegisteredWidgetParameterException
   *   Thrown when the parameter is not registered.
   */
  public function getParameterValue($parameterKey);

}
